// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter_test/flutter_test.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

void main() {
  const Email email = Email.dirty('email');
  const String passwordString = 'password';
  const Password password = Password.dirty(passwordString);

  group('SignUpState', () {
    test('supports value comparisons', () {
      expect(
        const SignUpState(
          data: FormData.empty(),
        ),
        const SignUpState(
          data: FormData.empty(),
        ),
      );
    });

    test('returns same object when no properties are passed', () {
      expect(
        const SignUpState(
          data: FormData.empty(),
        ).copyWith(),
        const SignUpState(
          data: FormData.empty(),
        ),
      );
    });

    test('returns object with updated status when status is passed', () {
      expect(
        const SignUpState(
          data: FormData.empty(),
        ).copyWith(status: FormStatus.pure),
        const SignUpState(
          data: FormData.empty(),
        ),
      );
    });

    test('returns object with updated email when email is passed', () {
      expect(
        const SignUpState(
          data: FormData.empty(),
        ).copyWith(email: email),
        const SignUpState(
          email: email,
          data: FormData.empty(),
        ),
      );
    });

    test('returns object with updated password when password is passed', () {
      expect(
        const SignUpState(
          data: FormData.empty(),
        ).copyWith(password: password),
        const SignUpState(
          password: password,
          data: FormData.empty(),
        ),
      );
    });

    test(
        'returns object with updated data'
        ' when data is passed', () {
      expect(
        const SignUpState(
          data: FormData.empty(),
        ).copyWith(
          data: const FormData(
            [
              FormInput(
                'field',
                Name.pure(),
              ),
            ],
          ),
        ),
        const SignUpState(
          data: FormData(
            [
              FormInput(
                'field',
                Name.pure(),
              ),
            ],
          ),
        ),
      );
    });
  });
}
