// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_form_bloc/src/enums/enums.dart';
import 'package:wyatt_form_bloc/src/form/form.dart';

/// {@template list_option}
/// Form input for a list input
/// {@endtemplate}
class ListOption<T> extends FormInputValidator<List<T>, ValidationError> {
  /// {@macro list_option}
  const ListOption.pure({List<T>? defaultValue})
      : super.pure(defaultValue ?? const []);

  /// {@macro list_option}
  const ListOption.dirty({List<T>? value}) : super.dirty(value ?? const []);

  ListOption select(T? v) {
    if (v == null) {
      return this;
    }
    if (value.contains(v)) {
      final List<T> newValue = List.from(value)..remove(v);
      return ListOption<T>.dirty(value: newValue);
    } else {
      final List<T> newValue = List.from(value)..add(v);
      return ListOption<T>.dirty(value: newValue);
    }
  }

  @override
  ValidationError? validator(List<T>? value) {
    return value?.isNotEmpty == true ? null : ValidationError.invalid;
  }
}
